type BeforeExitTasks = () => Promise<unknown> | unknown;

const beforeExitTasks: Set<BeforeExitTasks> = new Set();
export const beforeExit = {
  add: async (task: BeforeExitTasks) => {
    beforeExitTasks.add(task);
  },
  cancel: async (task: BeforeExitTasks) => {
    beforeExitTasks.delete(task);
  },
};

const gracefulExit = async () => {
  const result = await Promise.allSettled(
    Array.from(beforeExitTasks).map((t) => t())
  );
  process.exit(result.find((i) => i.status === "rejected") ? 1 : 0);
};

process.once("SIGTERM", gracefulExit);
process.once("SIGINT", gracefulExit);
