import { URL } from "url";

import { vault } from "@gtr/config";
import { onceAsync } from "@gtr/utils";
import Mirai from "mirai-ts";

import { beforeExit } from "./before-exit";

export interface ServerConf {
  verifyKey: string;
  server: string;
  qq: number;
}

export interface AccountConf {
  account: number;
  password: string;
}

const initMirai = async (accountName = "default") => {
  const serverConf = await vault.get<ServerConf>(
    "projects/yumemi/extensions/qq/mirai.infra"
  );
  const accountConfig = await vault.get<AccountConf>(
    `projects/yumemi/extensions/qq/accounts/${accountName}`
  );

  const mirai = new Mirai({
    adapters: ["http", "ws"],
    enableVerify: true,
    verifyKey: serverConf.verifyKey,
    singleMode: false,
    cacheSize: 4096,
    adapterSettings: {
      http: {
        host: "unknown",
        port: 1000,
        cors: ["*"],
      },
      ws: {
        host: "unknown",
        port: 1000,
        reservedSyncId: "-1",
      },
    },
  });

  const wsUrl = new URL(serverConf.server);
  wsUrl.protocol = wsUrl.protocol === "https:" ? "wss:" : "ws:";
  mirai.api.ws.address = wsUrl.toString();
  mirai.api.address = serverConf.server;

  mirai.api.axios.defaults.baseURL = serverConf.server;

  await mirai.link(accountConfig.account);

  beforeExit.add(() => mirai.release());

  return mirai;
};

const getMirai = onceAsync(() => initMirai());

export { initMirai, getMirai };
