import { GroupMessage } from "mirai-ts/dist/types/message-type";

import { config } from "../config";
import { redisStackBuilder } from "../infra";

const sysCache = redisStackBuilder.mkChild("system");

const sysAdmins = sysCache
  .json<number[]>("admins")
  .setDefaultValueProvider(() => []);

export interface GroupConfig {
  admins: number[];
  bans: number[];
}

const groupConfig = (groupId: number) =>
  sysCache
    .mkChild("groups")
    .json<GroupConfig>(groupId)
    .setDefaultValueProvider(() => ({
      admins: [],
      bans: [],
    }));

const isAdmin = async (msg: GroupMessage) => {
  const { admins } = await groupConfig(msg.sender.group.id).get();
  return !![admins, await sysAdmins.get(), config.staticAdmins].find((l) =>
    l.includes(msg.sender.id)
  );
};

export const sys = { isAdmin };
