import { RedisStackBuilder } from "@gtr-infra/redis";
import { FeedList } from "@gtr/random-bilibili-api";
import { delay } from "@gtr/utils";
import EventEmitter from "eventemitter3";

import { biliApi } from "../../common/biliapi";

const fetchInterval = 60 * 1000;

export const initFeedEvent = async (redis: RedisStackBuilder) => {
  redis = redis.mkChild("feed-list");

  const states = redis.json<FeedList.RoomInfo[]>("prev-states");
  const readyMark = redis.switch("ready-mark", fetchInterval * 2);

  const eventEmitter = new EventEmitter<{
    "live-start": [FeedList.RoomInfo];
    "live-stop": [FeedList.RoomInfo];
    "live-changed": [FeedList.RoomInfo, FeedList.RoomInfo];
  }>();

  const checkFeedList = async () => {
    const prevRooms = await states.get([]);
    const { list: curRooms } = await biliApi.feedList({ page: 0, size: 100 });
    await states.set(curRooms);

    const mark = await readyMark.get();
    readyMark.set(true);
    if (!mark) return;

    curRooms
      .filter((i) => !prevRooms.find((r) => r.roomid === i.roomid))
      .forEach((i) => eventEmitter.emit("live-start", i));

    prevRooms
      .filter((i) => !curRooms.find((r) => r.roomid === i.roomid))
      .forEach((i) => eventEmitter.emit("live-stop", i));

    const hash = (i: FeedList.RoomInfo) =>
      [i.area_id, i.cover, i.face, i.parent_area_id, i.pic, i.title].join(",");

    curRooms
      .map((i): [FeedList.RoomInfo | undefined, FeedList.RoomInfo] => [
        prevRooms.find((r) => r.roomid === i.roomid),
        i,
      ])
      .filter((pc): pc is [FeedList.RoomInfo, FeedList.RoomInfo] => !!pc[0])
      .filter(([prev, cur]) => hash(prev) !== hash(cur))
      .forEach(([prev, cur]) => eventEmitter.emit("live-changed", prev, cur));
  };

  const endlessCheck = async () => {
    for (;;) {
      try {
        await checkFeedList();
      } catch (e) {
        console.error(e);
      }
      await delay(fetchInterval);
    }
  };

  endlessCheck();

  return eventEmitter;
};
