import md5 from "md5";
import Mirai from "mirai-ts";

import { sys } from "../../common/system";
import { redisStackBuilder } from "../../infra";

const RepeatThreshold = 2;
const CooldownTime = 10 * 1000;

const repeaterGroups = redisStackBuilder.set("repeater:groups");

export const install = async (mirai: Mirai) => {
  const groups: Record<number, boolean> = await repeaterGroups
    .values()
    .then((l) => Object.fromEntries(l.map((g) => [g, true])));

  const addGroup = async (groupId: number) => {
    await repeaterGroups.add(`${groupId}`);
    groups[groupId] = true;
  };

  const removeGroup = async (groupId: number) => {
    await repeaterGroups.delete(`${groupId}`);
    groups[groupId] = false;
  };

  mirai.on("GroupMessage", async (msg) => {
    if (!msg.isAt()) return;
    if (!msg.plain.includes("复读")) return;
    if (!(await sys.isAdmin(msg))) return;

    const on = msg.plain.includes("开始");
    const off = !on && msg.plain.includes("停止");
    if (on === off) return;

    const groupId = msg.sender.group.id;

    if (on) {
      await addGroup(groupId);
      await msg.reply(msg.messageChain);
    }

    if (off) {
      await removeGroup(groupId);
      await msg.reply("ok~", true);
    }
  });

  const history: Record<number, string[]> = {};
  const cooldown: Record<number, boolean> = {};
  const lastRepeat: Record<number, string> = {};

  const groupCooldown = (groupId: number) => ({
    start() {
      cooldown[groupId] = true;
      setTimeout(() => (cooldown[groupId] = false), CooldownTime);
    },
    inProgress() {
      return !!cooldown[groupId];
    },
  });

  mirai.on("GroupMessage", async (msg) => {
    if (msg.get("At") || msg.get("AtAll")) return;
    if (msg.friend(mirai.qq)) return;
    const groupId = msg.sender.group.id;
    if (!groups[groupId]) return;

    const msgChain = msg.messageChain.slice(1).map((i) => {
      if (i.type === "Image") i.url = "";
      return i;
    });

    const hash = md5(JSON.stringify(msgChain));
    const arr = (history[groupId] = history[groupId] || []);
    arr.unshift(hash);
    arr.splice(RepeatThreshold, Infinity);

    const cd = groupCooldown(groupId);

    if (
      arr.length === RepeatThreshold &&
      new Set(arr).size === 1 &&
      lastRepeat[groupId] !== hash &&
      !cd.inProgress()
    ) {
      msg.reply(msgChain);
      lastRepeat[groupId] = hash;
      cd.start();
    }
  });
};
